@@ -29,6 +29,7 @@ module Ast_403 = Ast_403
 module Ast_404 = Ast_404
 module Ast_405 = Ast_405
 module Ast_406 = Ast_406
+module Ast_407 = Ast_407
 (*$*)
 
 (* A module for marshalling/unmarshalling arbitrary versions of Asts *)
@@ -47,6 +48,8 @@ module Migrate_404_405 = Migrate_parsetree_404_405
 module Migrate_405_404 = Migrate_parsetree_405_404
 module Migrate_405_406 = Migrate_parsetree_405_406
 module Migrate_406_405 = Migrate_parsetree_406_405
+module Migrate_406_407 = Migrate_parsetree_406_407
+module Migrate_407_406 = Migrate_parsetree_407_406
 (*$*)
 
 (* An abstraction of OCaml compiler versions *)
@@ -63,6 +66,7 @@ module OCaml_403 = Versions.OCaml_403
 module OCaml_404 = Versions.OCaml_404
 module OCaml_405 = Versions.OCaml_405
 module OCaml_406 = Versions.OCaml_406
+module OCaml_407 = Versions.OCaml_407
 (*$*)
 module OCaml_current = Versions.OCaml_current
 
