@@ -30,14 +30,19 @@ type cookies = (string, cookie) Hashtbl.t
 
 let create_cookies () = Hashtbl.create 3
 
+let global_cookie_table = create_cookies ()
+
 let get_cookie table name version =
   match
     match Hashtbl.find table name with
     | result -> Some result
     | exception Not_found ->
         match Ast_mapper.get_cookie name with
-        | None -> None
         | Some expr -> Some (Cookie ((module OCaml_current), expr))
+        | None ->
+            match Hashtbl.find global_cookie_table name with
+            | result -> Some result
+            | exception Not_found -> None
   with
   | None -> None
   | Some (Cookie (version', expr)) ->
@@ -46,6 +51,9 @@ let get_cookie table name version =
 let set_cookie table name version expr =
   Hashtbl.replace table name (Cookie (version, expr))
 
+let set_global_cookie name version expr =
+  set_cookie global_cookie_table name version expr
+
 let apply_cookies table =
   Hashtbl.iter (fun name (Cookie (version, expr)) ->
       Ast_mapper.set_cookie name
@@ -69,8 +77,19 @@ type 'types rewriter = config -> cookies -> 'types get_mapper
 type rewriter_group =
     Rewriters : 'types ocaml_version * (string * 'types rewriter) list -> rewriter_group
 
+let rewriter_group_names (Rewriters (_, l)) = List.map fst l
+
 let uniq_rewriter = Hashtbl.create 7
-let registered_rewriters = ref []
+module Pos_map = Map.Make(struct
+    type t = int
+    let compare : int -> int -> t = compare
+  end)
+let registered_rewriters = ref Pos_map.empty
+
+let all_rewriters () =
+  Pos_map.bindings !registered_rewriters
+  |> List.map (fun (_, r) -> !r)
+  |> List.concat
 
 let uniq_arg = Hashtbl.create 7
 let registered_args_reset = ref []
@@ -95,7 +114,7 @@ let () =
         ; pos_cnum  = 0
         };
       let expr = Parse.expression lexbuf in
-      Ast_mapper.set_cookie name expr
+      set_global_cookie name (module OCaml_current) expr
   in
   registered_args :=
     ("--cookie", Arg.String set_cookie,
@@ -118,7 +137,7 @@ let add_rewriter
   in
   add_rewriter
 
-let register ~name ?reset_args ?(args=[]) version rewriter =
+let register ~name ?reset_args ?(args=[]) ?(position=0) version rewriter =
   (* Validate name *)
   if name = "" then
     invalid_arg "Migrate_parsetree_driver.register: name is empty";
@@ -140,8 +159,18 @@ let register ~name ?reset_args ?(args=[]) version rewriter =
   | Some f -> registered_args_reset := f :: !registered_args_reset
   end;
   registered_args := List.rev_append args !registered_args;
-  registered_rewriters :=
-    add_rewriter Is_rewriter version name rewriter !registered_rewriters
+  let r =
+    try
+      Pos_map.find position !registered_rewriters
+    with Not_found ->
+      let r = ref [] in
+      registered_rewriters := Pos_map.add position r !registered_rewriters;
+      r
+  in
+  r := add_rewriter Is_rewriter version name rewriter !r
+
+let registered_args () = List.rev !registered_args
+let reset_args () = List.iter (fun f -> f ()) !registered_args_reset
 
 (** {1 Accessing or running registered rewriters} *)
 
@@ -190,7 +219,10 @@ let rec rewrite_signature
 
 let rewrite_signature config version sg =
   let cookies = create_cookies () in
-  let sg = rewrite_signature config cookies Signature version sg !registered_rewriters in
+  let sg =
+    rewrite_signature config cookies Signature version sg
+      (all_rewriters ())
+  in
   apply_cookies cookies;
   sg
 
@@ -219,19 +251,22 @@ let rec rewrite_structure
 
 let rewrite_structure config version st =
   let cookies = create_cookies () in
-  let st = rewrite_structure config cookies Structure version st !registered_rewriters in
+  let st =
+    rewrite_structure config cookies Structure version st
+      (all_rewriters ())
+  in
   apply_cookies cookies;
   st
 
 let run_as_ast_mapper args =
-  let spec = List.rev !registered_args in
+  let spec = registered_args () in
   let args, usage =
     let me = Filename.basename Sys.executable_name in
     let args = match args with "--as-ppx" :: args -> args | args -> args in
     (Array.of_list (me :: args),
      Printf.sprintf "%s [options] <input ast file> <output ast file>" me)
   in
-  List.iter (fun f -> f ()) !registered_args_reset;
+  reset_args ();
   match
     Arg.parse_argv args spec
       (fun arg -> raise (Arg.Bad (Printf.sprintf "invalid argument %S" arg)))
@@ -330,22 +365,28 @@ let load_file file =
         ; pos_cnum  = 0
         };
       Location.input_name := fn;
-      if Filename.check_suffix fn ".ml" then
+      match file with
+      | Impl fn ->
         (fn, Impl (Parse.implementation lexbuf))
-      else if Filename.check_suffix fn ".mli" then
-        (fn, Intf (Parse.interface lexbuf))
-      else
-        (* TODO: add support for -intf and -impl *)
-        Location.raise_errorf ~loc:(Location.in_file fn)
-          "I can't decide whether %s is an implementation or interface file"
-          fn)
+      | Intf fn ->
+        (fn, Intf (Parse.interface lexbuf)))
 
-let with_output output ~f =
+let with_output ?bin output ~f =
   match output with
-  | None -> f stdout
+  | None ->
+      begin match bin with
+      | Some bin -> set_binary_mode_out stdout bin
+      | None -> ()
+      end;
+      f stdout
   | Some fn -> with_file_out fn ~f
 
-let process_file ~config ~output ~dump_ast ~embed_errors file =
+type output_mode =
+  | Pretty_print
+  | Dump_ast
+  | Null
+
+let process_file ~config ~output ~output_mode ~embed_errors file =
   let fn, ast = load_file file in
   let ast =
     match ast with
@@ -378,59 +419,112 @@ let process_file ~config ~output ~dump_ast ~embed_errors file =
       in
       Impl (st, Ast_mapper.add_ppx_context_str ~tool_name:config.tool_name st)
   in
-  with_output output ~f:(fun oc ->
-    if dump_ast then begin
+  match output_mode with
+  | Dump_ast ->
+    with_output ~bin:true output ~f:(fun oc ->
       let ast =
         match ast with
         | Intf (_, sg) -> Ast_io.Intf ((module OCaml_current), sg)
         | Impl (_, st) -> Ast_io.Impl ((module OCaml_current), st)
       in
-      Ast_io.to_channel oc fn ast
-    end else begin
+      Ast_io.to_channel oc fn ast)
+  | Pretty_print ->
+    with_output output ~f:(fun oc ->
       let ppf = Format.formatter_of_out_channel oc in
       (match ast with
        | Intf (sg, _) -> Pprintast.signature ppf sg
        | Impl (st, _) -> Pprintast.structure ppf st);
-      Format.pp_print_newline ppf ()
-    end)
+      Format.pp_print_newline ppf ())
+  | Null ->
+    ()
+
+let print_transformations () =
+  let print_group name = function
+    | [] -> ()
+    | names ->
+        Printf.printf "%s:\n" name;
+        List.iter (Printf.printf "%s\n") names
+  in
+  all_rewriters ()
+  |> List.map rewriter_group_names
+  |> List.concat
+  |> print_group "Registered Transformations";
+  Ppx_derivers.derivers ()
+  |> List.map (fun (x, _) -> x)
+  |> print_group "Registered Derivers"
+
 
 let run_as_standalone_driver () =
+  let request_print_transformations = ref false in
   let output = ref None in
-  let dump_ast = ref false in
+  let output_mode = ref Pretty_print in
+  let output_mode_arg = ref "" in
   let files = ref [] in
   let embed_errors = ref false in
+  let embed_errors_arg = ref "" in
   let spec =
-    let as_ppx () =
-      raise (Arg.Bad "--as-ppx must be passed as first argument")
+    let fail fmt = Printf.ksprintf (fun s -> raise (Arg.Bad s)) fmt in
+    let incompatible a b = fail "%s and %s are incompatible" a b in
+    let as_ppx () = fail "--as-ppx must be passed as first argument" in
+    let set_embed_errors arg =
+      if !output_mode = Null then incompatible !output_mode_arg arg;
+      embed_errors := true;
+      embed_errors_arg := arg
+    in
+    let set_output_mode arg mode =
+      match !output_mode, mode with
+      | Pretty_print, _ ->
+        if mode = Null && !embed_errors then
+          incompatible !embed_errors_arg arg;
+        if mode = Null && !output <> None then
+          incompatible "-o" arg;
+        output_mode := mode;
+        output_mode_arg := arg
+      | _, Pretty_print -> assert false
+      | Dump_ast, Dump_ast | Null, Null -> ()
+      | _ -> incompatible !output_mode_arg arg
+    in
+    let set_output fn =
+      if !output_mode = Null then incompatible !output_mode_arg "-o";
+      output := Some fn
     in
     let as_pp () =
-      dump_ast := true;
-      embed_errors := true
+      let arg = "--as-pp" in
+      set_output_mode arg Dump_ast;
+      set_embed_errors arg
     in
     [ "--as-ppx", Arg.Unit as_ppx,
       " Act as a -ppx rewriter"
     ; "--as-pp", Arg.Unit as_pp,
       " Shorthand for: --dump-ast --embed-errors"
-    ; "--dump-ast", Arg.Set dump_ast,
+    ; "--dump-ast", Arg.Unit (fun () -> set_output_mode "--dump-ast" Dump_ast),
       " Output a binary AST instead of source code"
-    ; "-o", Arg.String (fun o -> output := Some o),
+    ; "--null", Arg.Unit (fun () -> set_output_mode "--null" Null),
+      " Output nothing, just report errors"
+    ; "-o", Arg.String set_output,
       "FILE Output to this file instead of the standard output"
     ; "--intf", Arg.String (fun fn -> files := Intf fn :: !files),
       "FILE Treat FILE as a .mli file"
     ; "--impl", Arg.String (fun fn -> files := Impl fn :: !files),
       "FILE Treat FILE as a .ml file"
-    ; "--embed-errors", Arg.Set embed_errors,
+    ; "--embed-errors", Arg.Unit (fun () -> set_embed_errors "--embed-errors"),
       " Embed error reported by rewriters into the AST"
+    ; "--print-transformations", Arg.Set request_print_transformations,
+      " Print registered transformations in their order of executions"
     ]
   in
-  let spec = Arg.align (spec @ List.rev !registered_args) in
+  let spec = Arg.align (spec @ registered_args ()) in
   let me = Filename.basename Sys.executable_name in
   let usage = Printf.sprintf "%s [options] [<files>]" me in
   try
-    List.iter (fun f -> f ()) !registered_args_reset;
+    reset_args ();
     Arg.parse spec (fun anon -> files := guess_file_kind anon :: !files) usage;
+    if !request_print_transformations then begin
+      print_transformations ();
+      exit 0
+    end;
     let output = !output in
-    let dump_ast = !dump_ast in
+    let output_mode = !output_mode in
     let embed_errors = !embed_errors in
     let config =
       (* TODO: we could add -I, -L and -g options to populate these fields. *)
@@ -442,7 +536,8 @@ let run_as_standalone_driver () =
       ; extras       = []
       }
     in
-    List.iter (process_file ~config ~output ~dump_ast ~embed_errors) (List.rev !files)
+    List.iter (process_file ~config ~output ~output_mode ~embed_errors)
+      (List.rev !files)
   with exn ->
     Location.report_exception Format.err_formatter exn;
     exit 1
@@ -452,7 +547,7 @@ let run_as_ppx_rewriter () =
   let n = Array.length a in
   if n <= 2 then begin
     let me = Filename.basename Sys.executable_name in
-    Arg.usage (List.rev !registered_args)
+    Arg.usage (registered_args ())
       (Printf.sprintf "%s [options] <input ast file> <output ast file>" me);
     exit 2
   end;
