@@ -676,6 +676,13 @@ module OCaml_406 = struct
   let string_version = "4.06"
 end
 let ocaml_406 : OCaml_406.types ocaml_version = (module OCaml_406)
+module OCaml_407 = struct
+  module Ast = Ast_407
+  include Make_witness(Ast_407)
+  let version = 407
+  let string_version = "4.07"
+end
+let ocaml_407 : OCaml_407.types ocaml_version = (module OCaml_407)
 (*$*)
 
 let all_versions : (module OCaml_version) list = [
@@ -686,6 +693,7 @@ let all_versions : (module OCaml_version) list = [
   (module OCaml_404 : OCaml_version);
   (module OCaml_405 : OCaml_version);
   (module OCaml_406 : OCaml_version);
+  (module OCaml_407 : OCaml_version);
   (*$*)
 ]
 
@@ -702,6 +710,8 @@ include Register_migration(OCaml_404)(OCaml_405)
     (Migrate_parsetree_404_405)(Migrate_parsetree_405_404)
 include Register_migration(OCaml_405)(OCaml_406)
     (Migrate_parsetree_405_406)(Migrate_parsetree_406_405)
+include Register_migration(OCaml_406)(OCaml_407)
+    (Migrate_parsetree_406_407)(Migrate_parsetree_407_406)
 (*$*)
 
 module OCaml_current = OCaml_OCAML_VERSION
